/*

* FreeRTOS for AVR Small demo / Mega Pack V02 *
  

http://avr.paslog.jp/article/1157172.html
V02: 2010/04/11 by audin
V01: 2009/06/27 by audin
  
*/

#include <avr/pgmspace.h>
#include "FreeRTOS.h"
#include "task.h"
#include "stdio.h"
#include "time_def.h"
#include "sio_drv.h"
#include "sio_intr.h"

#define mBRate(baud)  ((F_CPU)/16/(baud))-1			// UART

//static void sio_set(unsigned int baud);

void __io_putchar ( char c);
int __io_putchar_for_rprintf( int c );
int __io_putchar_for_stdout(char c , FILE *stream);
static FILE mystdout = FDEV_SETUP_STREAM( __io_putchar_for_stdout, NULL, _FDEV_SETUP_WRITE );

/*******************

* sio_set
  
 *******************/
static void sio_set(uint16_t baud)
{
	UBRR0H = (unsigned char)(baud>>8);	// {[[g
	UBRR0L = (unsigned char)baud;		// {[[g
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);		// M
#if defined( UCSRC )
	UCSRC = (1<<URSEL)|(3<<UCSZ0)|(0<<USBS)|(0<<UPM0);
#endif
#if defined( UCSR0C )
	UCSR0C = (3<<UCSZ00)|(0<<USBS0)|(0<<UPM00);
#endif

}

/*******************

* sioDrv_init
  
 *******************/
static uint32_t uBaudrate;
void sioDrv_init(uint32_t baud)
{
  sio_set( mBRate(baud) );
  uBaudrate = baud;
  stdout = &mystdout;
}

/*******************

* sioDrv_GetBaudRate
  
 *******************/
uint32_t sioDrv_GetBaudRate(void)
{
	return uBaudrate;
}

/*******************

* sio_puts
  
 *******************/
void sioDrv_puts(PGM_P buf)
{
	char c;
	while( c = pgm_read_byte(buf++) ){
		__io_putchar( c );
	}
}

/*******************

* __io_putchar    : for small printf
  
 *******************/
void __io_putchar ( char c)
{
	if( c == '\n' ){
		__io_putchar('\r');
	}
	while ( ! isUART0_TxRx_FIFO_empty() );// Mޯ̧󂫂mF
	TX_REG = c;
	vTaskDelay( MSEC(70) );
}

/*******************

* __io_putchar_for_stdout
  
 *******************/
int __io_putchar_for_stdout(char c , FILE *stream)
{
	__io_putchar (c);
	return 0;
}

/*******************

* cSio_getchar
  
 *******************/
char sioDrv_getchar(void)
{
	char c = 0;
	sioIntr_rxIntr_disable();
	if( sioIntr_isRxDataExist() ){
		c = sioIntr_getRxData();
	}
	sioIntr_rxIntr_enable();
	return c;
}


