/*

* FreeRTOS for AVR Small demo / Mega Pack V02 *
  
http://avr.paslog.jp/article/1157172.html
V02: 2010/04/11 by audin
V01: 2009/06/27 by audin
  
*/


#include "FreeRTOS.h"
#include "task.h"
#include "sio_drv.h"
#include "sio_tsk.h"
#include "sio_intr.h"
#include "platform.h"
#include "stdio.h"

#if defined( RPRINTF )
#include "rprintf.h"
#undef printf
#define printf rprintf
#endif

static unsigned char state=0;

const char PROGMEM msg_clr[]	= /*  */ "\x1b[2J";	//clear 
const char PROGMEM msg_os[]		= /*  */ "FreeRTOS %s for AVR, UART Transfer test. \n";
const char PROGMEM msg_cpu[]	= /*  */ "CPU may be ATmega328p or something.\n";
const char PROGMEM msg_wel[]	= /*  */ "Welcome !\n";
const char PROGMEM msg_bps[]	= /*  */ "UART speed is %lu bps.\n"  ;
const char PROGMEM msg_led[]	= /*  */ "Now LED blink speed is %d.\n"  ;
const char PROGMEM msg_in[]		= /*  */ "\nPlease input LED blink speed [1 - 9]\n"  ;


static portTickType led_blink_speed = 9;


//#define sioDrv_puts printf_P

/*******************

* sio_Task
  
 *******************/
extern int max_val;
void sio_tsk( void *pvParameters )
{
	( void ) pvParameters;	// not used
	sioIntr_rxIntr_enable();

	while(1){
		switch(state++){
			case 0:
				printf_P( msg_clr );				//clear
				vTaskDelay( SEC(2) );
				printf_P( msg_wel );
				vTaskDelay( SEC(2) );
				break;
			case 1:
				printf_P( msg_os, tskKERNEL_VERSION_NUMBER );
				vTaskDelay( SEC(2) );
				printf_P( msg_cpu );
				vTaskDelay( SEC(2) );
				break;
			case 2:
				printf_P( msg_bps, sioDrv_GetBaudRate() );
				vTaskDelay( SEC(2) );
				break;
			case 3:
				printf_P( msg_led, (int)sio_get_led_blink_speed() );
				printf_P( msg_in );
				vTaskDelay( SEC( 3 ) );
				state = 0;
				break;
		}
	}
}

/*******************

* sio_Key_Input_tsk
  
 *******************/
void sio_Key_Input_tsk( void *pvParameters )
{
	char c;
	( void ) pvParameters;	// not used

	while(1){
		vTaskSuspend( NULL );		// TXyh̎iUARTM݂ŃW[j
		c = sioDrv_getchar();
		if( (c  >= '1') && ('9' >= c ) ){
			led_blink_speed = c - '0';
		}
	}
}

/*******************

* sio_get_led_blink_speed
  
 *******************/
portTickType sio_get_led_blink_speed(void)
{
	
	return led_blink_speed;
}

/*******************

* sio_init
  
 *******************/
void sio_init( uint32_t baudrate)
{
  sioDrv_init( baudrate );
}
